@@ -31,22 +31,21 @@ from .version import __version__
 def main():
     """Implementation of MWP analysis on C code in Python."""
     parser = argparse.ArgumentParser(prog='pymwp', description=main.__doc__)
-    args = _parse_args(parser)
+    args = __parse_args(parser)
 
     if not args.file:
         parser.print_help()
         sys.exit(1)
 
-    log_level = 0 if args.silent else 40
-    log_filename = args.logfile
-    setup_logger(logging.FATAL - log_level, log_filename=log_filename)
+    log_level = logging.FATAL - (0 if args.silent else 40)
+    __setup_logger(log_level, args.logfile)
     file_out = args.out or default_file_out(args.file)
 
     ast = parse(args.file, not args.no_cpp, args.cpp, args.cpp_args)
     Analysis.run(ast, file_out, args.no_save)
 
 
-def _parse_args(
+def __parse_args(
         parser: argparse.ArgumentParser,
         args: Optional[List] = None) -> argparse.Namespace:
     """Setup available program arguments."""
@@ -101,9 +100,9 @@ def _parse_args(
     return parser.parse_args(args)
 
 
-def setup_logger(
-        level: int = logging.ERROR,
-        log_filename: Optional[str] = None) -> logging.Logger:
+def __setup_logger(
+        level: int = logging.ERROR, log_filename: Optional[str] = None
+) -> None:
     """Create a configured instance of logger.
 
     Arguments:
