@@ -1,7 +1,8 @@
 from pymwp import Analysis, Polynomial
 from .mocks.ast_mocks import \
     INFINITE_2C, NOT_INFINITE_2C, IF_WO_BRACES, IF_WITH_BRACES, \
-    VARIABLE_IGNORED, OTHER_BRACES_ISSUES, BASICS_ASSIGN_VALUE, PARAMS
+    VARIABLE_IGNORED, OTHER_BRACES_ISSUES, BASICS_ASSIGN_VALUE, PARAMS, \
+    FUNCTION_CALL
 
 
 def test_analyze_infinite2():
@@ -59,14 +60,14 @@ def test_analyze_variable_ignore():
                             [2, 0], [2, 1], [2, 2]]
     assert set(relation.variables) == {'X2', 'X3', 'X1', 'X4'}
 
-    wmp = '+w.delta(0,0)+m.delta(1,0)+p.delta(2,0)'
-    wpm = '+w.delta(0,0)+p.delta(1,0)+m.delta(2,0)'
+    mpw = '+m.delta(0,0)+p.delta(1,0)+w.delta(2,0)'
+    pmw = '+p.delta(0,0)+m.delta(1,0)+w.delta(2,0)'
     o = '+o'
     m = '+m'
     res = [
-        [m, wpm, o, wpm],
+        [m, pmw, o, pmw],
         [o, o, o, o],
-        [o, wmp, m, wmp],
+        [o, mpw, m, mpw],
         [o, o, o, o],
     ]
 
@@ -109,3 +110,15 @@ def test_analysis_identifies_function_params():
     relation = Analysis.run(PARAMS, no_save=True)[0]
 
     assert set(relation.variables) == {'x1', 'x2', 'x3'}
+
+
+def test_analysis_returns_all_functions():
+    """If input file contains multiple functions result contains
+    evaluation of each function (example 5)
+    """
+    result = Analysis.run(FUNCTION_CALL, no_save=True)
+    f, _, _ = result['f']
+    foo, _, _ = result['foo']
+
+    assert set(f.variables) == {'X1', 'X2'}
+    assert set(foo.variables) == {'X1', 'X2', 'X3'}
